
#include "sys.h"
#ifndef WINDOWS
	#include <time.h>
#endif

//[of]:getMonotonicTime
ENTRY uint64_t csys_getMonotonicTime()
{
	#ifdef WINDOWS

		LARGE_INTEGER v;
		LARGE_INTEGER freq;

		QueryPerformanceCounter(&v);
		QueryPerformanceFrequency(&freq);
	
		return (uint64_t)v.QuadPart * 1000000 / (uint64_t)freq.QuadPart;
		
	#else

		struct timespec now;
		if( clock_gettime(CLOCK_MONOTONIC, &now) != 0 )
			return 0;
		return (now.tv_sec * 1000000 + now.tv_nsec / 1000);

	#endif
}
//[cf]

#ifdef WINDOWS
//[of]:	windowsTimeToTime
ENTRY uint64_t csys_windowsTimeToTime(uint64_t time)
{
	uint64_t d;
	uint64_t t;
	#ifdef _MSC_VER
		uint64_t microsecondsPerDay = 24i64 * 60i64 * 60i64 * 1000i64 * 1000i64;
	#else
		uint64_t microsecondsPerDay = 24ULL * 60ULL * 60ULL * 1000ULL * 1000ULL;
	#endif
	
	// Convert to microseconds
	t = time / 10;

	// Extract the number of days
	d = t / microsecondsPerDay;

	// Convert to julian day
	d += 2305814;

	// Extract time
	t %= microsecondsPerDay;

	return d * microsecondsPerDay + t;
}
//[cf]
#else
//[of]:	unixTimeToTime
ENTRY uint64_t csys_unixTimeToTime(uint64_t time)
{
	uint64_t t;
	t = time;
	t += 2440588ULL * 86400ULL;
	t *= 1000000ULL;
	return t;
}
//[cf]
#endif
